/*
  The oRTP LinPhone RTP library intends to provide basics for a RTP stack.
  Copyright (C) 2001  Simon MORLAT simon.morlat@linphone.org

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include "payloadtype.h"

char offset127=127;
char offset0=0;

PayloadType pcmu8000={
	PAYLOAD_AUDIO_CONTINUOUS,
	8000,
	1.0,
	&offset127,
	1,
	64000,
	"PCMU/8000/1"
};

PayloadType pcma8000={
	PAYLOAD_AUDIO_CONTINUOUS,
	8000,
	1.0,
	&offset127,
	1,
	64000,
	"PCMA/8000/1"
};

PayloadType pcm8000={
	PAYLOAD_AUDIO_CONTINUOUS,
	8000,
	2.0,
	&offset0,
	1,
	128000,
	"PCM/8000/1"
};

PayloadType lpc1016={
	PAYLOAD_AUDIO_PACKETIZED,
	8000,
	0,
	NULL,
	0,
	2400,
	"1016/8000/1"
};


PayloadType gsm=
{
	PAYLOAD_AUDIO_PACKETIZED,
	8000,
	0,
	NULL,
	0,
	13500,
	"GSM/8000/1"
};


PayloadType mpv=
{
	PAYLOAD_VIDEO,
	90000,
	0,
	NULL,
	0,
	0,
	"MPV/90000/1"
};


PayloadType h261={
	PAYLOAD_VIDEO,
	90000,
	0,
	NULL,
	0,
	0,
	"h261/90000/1"
};

RtpProfile av_profile;


void av_profile_init(RtpProfile *profile)
{
	rtp_profile_clear_all(profile);
	rtp_profile_set_name(profile,"AV profile");
	rtp_profile_set_payload(profile,0,&pcmu8000);
	rtp_profile_set_payload(profile,1,&lpc1016);
	rtp_profile_set_payload(profile,3,&gsm);
	rtp_profile_set_payload(profile,8,&pcma8000);
	rtp_profile_set_payload(profile,31,&h261);
	rtp_profile_set_payload(profile,32,&mpv);
}
	

/**
 *rtp_profile_clear_all:
 *@profile: an RTP profile (#RtpProfile object)
 *
 *	Initialize the profile to the empty profile (all payload type are unassigned).
 *
**/

/**
 *rtp_profile_set_name:
 *@profile: 	a rtp profile object (#RtpProfile)
 *@nm:			a string
 *
 *	Set a name to the rtp profile. (This is not required)
 *
**/

/**
 *rtp_profile_get_name:
 *@profile:		a rtp profile object (#RtpProfile)
 *
 *Returns:	the name of the rtp profile. May be NULL.
**/

/**
 *rtp_profile_set_payload:
 *@profile: an RTP profile (a #RtpProfile object)
 *@index:	the payload type number
 *@pt:		the payload type description (a #PayloadType object )
 *
 *	Assign payload type number @index to payload type desribed in @pt for the RTP profile
 *	@profile.
 *
**/

/**
 *rtp_profile_get_payload:
 *@profile:	an RTP profile (a #RtpProfile object)
 *@index:	the payload type number
 *
 *	Gets the payload description of the payload type @index in the profile @profile.
 *
 *Returns: the payload description (a #PayloadType object)
**/

/**
 *rtp_profile_clear_payload:
 *@profile:	an RTP profile (a #RtpProfile object)
 *@index:	the payload type number
 *
 *	Set payload type number @index unassigned in profile @profile.
 *
**/